@@ -1,4 +1,3 @@
-use crate::logical_plan::iterator::ArenaExprIter;
 use crate::logical_plan::Context;
 use crate::prelude::*;
 use crate::utils::rename_field;
@@ -70,10 +69,11 @@ pub enum AExpr {
         truthy: Node,
         falsy: Node,
     },
-    Udf {
-        input: Node,
+    Function {
+        input: Vec<Node>,
         function: NoEq<Arc<dyn SeriesUdf>>,
         output_type: Option<DataType>,
+        collect_groups: bool,
     },
     Shift {
         input: Node,
@@ -81,7 +81,7 @@ pub enum AExpr {
     },
     Window {
         function: Node,
-        partition_by: Node,
+        partition_by: Vec<Node>,
         order_by: Option<Node>,
     },
     Wildcard,
@@ -289,12 +289,12 @@ impl AExpr {
                 Ok(Field::new(field.name(), data_type.clone()))
             }
             Ternary { truthy, .. } => arena.get(*truthy).to_field(schema, ctxt, arena),
-            Udf {
+            Function {
                 output_type, input, ..
             } => match output_type {
-                None => arena.get(*input).to_field(schema, ctxt, arena),
+                None => arena.get(input[0]).to_field(schema, ctxt, arena),
                 Some(output_type) => {
-                    let input_field = arena.get(*input).to_field(schema, ctxt, arena)?;
+                    let input_field = arena.get(input[0]).to_field(schema, ctxt, arena)?;
                     Ok(Field::new(input_field.name(), output_type.clone()))
                 }
             },
@@ -318,10 +318,12 @@ impl AExpr {
     }
 
     /// Check if AExpr equality. The nodes may differ.
-    /// j
+    ///
     /// For instance: there can be two columns "foo" in the memory arena. These are equal,
     /// but would have different node values.
+    #[cfg(feature = "private")]
     pub(crate) fn eq(node_left: Node, node_right: Node, expr_arena: &Arena<AExpr>) -> bool {
+        use crate::logical_plan::iterator::ArenaExprIter;
         let cmp = |(node_left, node_right)| {
             use AExpr::*;
             match (expr_arena.get(node_left), expr_arena.get(node_right)) {
