@@ -13,6 +13,7 @@ fn init_set() -> HashSet<Arc<String>, RandomState> {
 }
 
 /// utility function to get names of the columns needed in projection at scan level
+#[cfg(any(feature = "parquet", feature = "csv-file"))]
 fn get_scan_columns(
     acc_projections: &mut Vec<Node>,
     expr_arena: &Arena<AExpr>,
@@ -321,6 +322,7 @@ impl ProjectionPushDown {
                 };
                 Ok(lp)
             }
+            #[cfg(feature = "csv-file")]
             CsvScan {
                 path,
                 schema,
